# node 原生模块下载失败问题


<!--more-->

## 使用 http 模块下载 https 的链接会报异常

这时候需要使用 https 模块下载，下面是兼容 http 和 https 的下载函数：

```js
const fs = require("fs");
const path = require("path");
const http = require("http");
const https = require("https");

async function download(url, filePath) {
  const proto = url.startsWith("https") ? https : http;

  return new Promise((resolve, reject) => {
    const file = fs.createWriteStream(filePath);
    let fileInfo = null;

    // const request = proto.get(url, response => {
    const request = proto.get(url, (response) => {
      if (response.statusCode !== 200) {
        fs.unlink(filePath, () => {
          reject(new Error(`Failed to get '${url}' (${response.statusCode})`));
        });
        return;
      }

      fileInfo = {
        mime: response.headers["content-type"],
        size: parseInt(response.headers["content-length"], 10),
      };

      response.pipe(file);
    });

    // The destination stream is ended by the time it's called
    file.on("finish", () => resolve(fileInfo));

    request.on("error", (err) => {
      fs.unlink(filePath, () => reject(err));
    });

    file.on("error", (err) => {
      fs.unlink(filePath, () => reject(err));
    });

    request.end();
  });
}

const url = "https://100.100.22.140/files/1.0.zip";

download(url, path.resolve("./download"));
```

## 目标下载地址是自签的 https 证书，下载会报异常

但是如果目标下载地址是自签的 https 证书，则下载也会失败，这时候有两种解决方式：

第一种：环境变量设置 `NODE_TLS_REJECT_UNAUTHORIZED`

```js
process.env["NODE_TLS_REJECT_UNAUTHORIZED"] = 0;
```

第二种：https 模块下载时加上 `{ rejectUnauthorized: false }` 可选项

具体的例子如下：

```js
const fs = require("fs");
const path = require("path");
const http = require("http");
const https = require("https");

// process.env['NODE_TLS_REJECT_UNAUTHORIZED'] = 0;

async function download(url, filePath) {
  const proto = url.startsWith("https") ? https : http;

  return new Promise((resolve, reject) => {
    const file = fs.createWriteStream(filePath);
    let fileInfo = null;

    // const request = proto.get(url, response => {
    const request = proto.get(
      url,
      { rejectUnauthorized: false },
      (response) => {
        if (response.statusCode !== 200) {
          fs.unlink(filePath, () => {
            reject(
              new Error(`Failed to get '${url}' (${response.statusCode})`)
            );
          });
          return;
        }

        fileInfo = {
          mime: response.headers["content-type"],
          size: parseInt(response.headers["content-length"], 10),
        };

        response.pipe(file);
      }
    );

    // The destination stream is ended by the time it's called
    file.on("finish", () => resolve(fileInfo));

    request.on("error", (err) => {
      fs.unlink(filePath, () => reject(err));
    });

    file.on("error", (err) => {
      fs.unlink(filePath, () => reject(err));
    });

    request.end();
  });
}

const url = "https://100.100.22.140/files/1.0.zip";

download(url, path.resolve("./download"));
```

#### 参考链接

- [github issue](https://github.com/request/request/issues/418#issuecomment-17149236)
- [stackoverflow](https://stackoverflow.com/questions/23601989/client-certificate-validation-on-server-side-depth-zero-self-signed-cert-error)

